// Fragment Shader  file "gaussianShader.frag"

#version 330 core
#extension GL_ARB_explicit_uniform_location : enable

//This shader filters a given texture using a gaussian kernel
//Has to be called twice: 1.Vertical filtering, 2.Horizontal filtering
//This can be done because Gaussian is seperable! To improve performance

layout(location = 9) uniform sampler2D colorMap;
//direction is vec2(0, 1.0/height_colorMap) for vertical blur and vec2(1.0/width_colorMap, 0 ) for horizontal blur. 
layout(location = 42) uniform vec2 direction;  

in vec2 texCoords;

//7x1 gaussian kernel (x = offset, y = weight)
//const vec2 gaussian[4] = { 
//	vec2(0.0,	0.3293523843),
//	vec2(1.0,	0.2347277890),
//	vec2(2.0,	0.0849719524),
//	vec2(3.0,	0.0156240665)
//};

   
//9x1 gaussian kernel
//const vec2 gaussian[5] = { 
//0.0, 0.2319187149
//1.0, 0.1964261243
//2.0, 0.1193409808
//3.0, 0.0520123961
//4.0, 0.0162611413
//}

//9x1 gaussian kernel, when using billinear filtering
//sample at 0, between 1,2 and 3,4 (and -1,-2 and -3,-4)
//const vec2 gaussian[3] = { 
//	vec2(0.0,			0.2319187149),
//	vec2(1.3779398768,	0.3157671051),  
//	vec2(3.2381763406,	0.0682735374),
//};
//0.3157671051 = 0.1964261243 + 0.1193409808
//1.3779398768 = (1 * 0.1964261243 + 2 * 0.1193409808)/0.3157671051

//0.0682735374 = 0.0520123961 + 0.0162611413
//3.2381763406 = (3 * 0.0520123961 + 4 * 0.0162611413)/0.0682735374



//5x1 gaussian kernel
//const vec2 gaussian[3] = { 
// 0.0, 0.2326376277
// 1.0, 0.2147515969
// 2.0, 0.1689295893
//}

//5x1 gaussian kernel, when using billinear filtering
//sample at 0, and between 1,2 and -1,-2
uniform vec2 gaussian[2] = { 
	vec2(0.0,			0.2326376277),
	vec2(1.4402863507,	0.3836811862)  
};
//0.3836811862 = 0.2147515969 + 0.1689295893
//1.4402863507 = (1 * 0.2147515969 + 2 * 0.1689295893)/0.3836811862


out vec4 out_color; //final color of fragment

void main() {

	out_color = texture(colorMap, texCoords)*gaussian[0].y;

	//5x1 gaussian kernel, with billinear filtering -> 3 texture lookups in total
	for(int i=1; i<2; i++) {
		out_color += texture(colorMap, texCoords + vec2( gaussian[i].x*direction.x, gaussian[i].x*direction.y))*gaussian[i].y;
		out_color += texture(colorMap, texCoords - vec2( gaussian[i].x*direction.x, gaussian[i].x*direction.y))*gaussian[i].y;
	}

}