// Vertex Shader  file "parallaxShader.vert"

#version 330 core
#extension GL_ARB_explicit_uniform_location : enable

//Blinn-Phong Shading
//with three light sources (1. point light, 2-3. spot light)

//per vertex attributes
layout(location = 1) in vec4 in_position;		//in local coordinates
layout(location = 2) in vec3 in_normal;
layout(location = 3) in vec3 in_color;
layout(location = 4) in vec2 in_texCoords;
layout(location = 8) in vec3 in_tangent;
layout(location = 12) in vec3 in_bitangent;

//light uniforms
layout(location = 6) uniform mat3 lightPosition;  //in world coordinates
layout(location = 7) uniform mat3 lightColor;
layout(location = 29) uniform mat3 attenuation;
//layout(location = 22) uniform mat3 spotDirection; //in which direction the spot light shines, in world coordinates


//geometry uniforms
layout(location = 5) uniform mat4 mvpMatrix;
layout(location = 10) uniform mat3 normalMatrix;
layout(location = 13) uniform mat4 modelMatrix;

//camera uniforms
layout(location = 14) uniform vec4 cameraPos;		//in world coordinates
layout(location = 15) uniform mat4 texMat_model1;
layout(location = 16) uniform mat4 texMat_model2;
layout(location = 17) uniform mat4 texMat_model3;

//layout(location = 44) uniform sampler2D normalMap;
//layout(location = 45) uniform sampler2D heightMap;


//variables for fragment shader		
out mat3 lightIntensity;	//intensity of light, includes color
out mat3 lightDirection;	//direction vertex to light
out mat3 lightDirectionWorld;
//out mat3 spotDirection_tangent;
out vec3 cameraDirection;  //direction vertex to camera
out vec3 normal;		   //transformed normal vector of vertex
out vec2 texCoords;
out vec3 color;
out vec4 SM_tex_coords1;
out vec4 SM_tex_coords2;
out vec4 SM_tex_coords3;

void main(void) {

	//mat3 tbn = transpose(mat3(in_tangent, in_bitangent, in_normal));
	normal = normalMatrix * normalize(in_normal);
	vec3 tangent = normalMatrix * normalize(in_tangent);
	vec3 bitangent = normalMatrix * normalize(in_bitangent);
	
	//mat3 tbn = mat3(in_tangent, in_bitangent, in_normal);
	mat3 tbn = transpose(mat3(tangent,bitangent,normal));
	
	SM_tex_coords1 = texMat_model1*in_position;
	SM_tex_coords2 = texMat_model2*in_position;
	SM_tex_coords3 = texMat_model3*in_position;


	texCoords = in_texCoords;
	color = in_color;
	//normal in tangent space
	//normal =   normalMatrix * in_normal;

	//used just for testing purposes
	normal = normalize(tbn * normal);
	//normal = normalize(in_tangent);
	//normal =   normalMatrix * in_normal;

	vec4 worldPos = modelMatrix * in_position;

	cameraDirection = vec3(cameraPos-worldPos);

	lightDirectionWorld[0] = lightPosition[0] - worldPos.xyz;
	float distanceToLight = length(lightDirectionWorld[0]);
	lightIntensity[0] = lightColor[0]*(1/(attenuation[0].x + attenuation[0].y*distanceToLight + attenuation[0].z*distanceToLight*distanceToLight));
	
	lightDirectionWorld[1] = lightPosition[1] - worldPos.xyz;
	distanceToLight = length(lightDirectionWorld[1]);
	lightIntensity[1] = lightColor[1]*(1/(attenuation[1].x + attenuation[1].y*distanceToLight + attenuation[1].z*distanceToLight*distanceToLight));

	lightDirectionWorld[2] = lightPosition[2] - worldPos.xyz;
	distanceToLight = length(lightDirectionWorld[2]);
	lightIntensity[2] = lightColor[2]*(1/(attenuation[2].x + attenuation[2].y*distanceToLight + attenuation[2].z*distanceToLight*distanceToLight));
	
	//transform light direction and eye vector to tangent space (normalization is done in fragment shader, do i need to normalize twice?)
	//need to transform to object space first (multiply with inverse model matrix)
	//mat4 invModel = inverse(modelMatrix); //should be precomputed
	//lightDirection[0] = normalize(vec3(vec4(lightDirection[0],0.0f) * invModel));
	//lightDirection[1] = normalize(vec3(vec4(lightDirection[1],0.0f) * invModel));
	//lightDirection[2] = normalize(vec3(vec4(lightDirection[2],0.0f) * invModel));
	//lightDirection[0] = normalize(tbn * lightDirection[0]);
	//lightDirection[1] = normalize(tbn * lightDirection[1]);
	//lightDirection[2] = normalize(tbn * lightDirection[2]);
	//cameraDirection = normalize(vec3(vec4(cameraDirection,0.0f) * invModel));
	//cameraDirection = normalize(tbn * cameraDirection);

	lightDirection[0] = tbn * lightDirectionWorld[0];
	lightDirection[1] = tbn * lightDirectionWorld[1];
	lightDirection[2] = tbn * lightDirectionWorld[2];
	cameraDirection = tbn * cameraDirection;
	//spotDirection_tangent[0] = tbn * spotDirection[0];
	//spotDirection_tangent[1] = tbn * spotDirection[1];
	//spotDirection_tangent[2] = tbn * spotDirection[2];

	//According to slides: compute normalized half vector - should that be computed in the vertex shader instead of the fragement shader?


	gl_Position = mvpMatrix * in_position;

}
