// Vertex Shader  file "phongShader.vert"

#version 330 core
#extension GL_ARB_explicit_uniform_location : enable

//Blinn-Phong Shading
//with three light sources (1. point light, 2-3. spot light)

//per vertex attributes
layout(location = 1) in vec4 in_position;		//in local coordinates
layout(location = 2) in vec3 in_normal;
layout(location = 3) in vec3 in_color;
layout(location = 4) in vec2 in_texCoords;

//light uniforms
layout(location = 6) uniform mat3 lightPosition;  //in world coordinates
layout(location = 7) uniform mat3 lightColor;
layout(location = 29) uniform mat3 attenuation;

//geometry uniforms
layout(location = 5) uniform mat4 mvpMatrix;
layout(location = 10) uniform mat3 normalMatrix;
layout(location = 13) uniform mat4 modelMatrix;

//camera uniforms
layout(location = 14) uniform vec4 cameraPos;		//in world coordinates
layout(location = 15) uniform mat4 texMat_model1;
layout(location = 16) uniform mat4 texMat_model2;
layout(location = 17) uniform mat4 texMat_model3;


//variables for fragment shader		
out mat3 lightIntensity;	//intensity of light, includes color
out mat3 lightDirection;	//direction vertex to light
out vec3 cameraDirection;  //direction vertex to camera
out vec3 normal;		   //transformed normal vector of vertex
out vec2 texCoords;
out vec3 color;
out vec4 SM_tex_coords1;
out vec4 SM_tex_coords2;
out vec4 SM_tex_coords3;

void main(void) {

	SM_tex_coords1 = texMat_model1*in_position;
	SM_tex_coords2 = texMat_model2*in_position;
	SM_tex_coords3 = texMat_model3*in_position;

	texCoords = in_texCoords;
	color = in_color;
	normal =   normalMatrix * in_normal;

	vec4 worldPos = modelMatrix * in_position;

	cameraDirection = vec3(cameraPos-worldPos);

	lightDirection[0] = lightPosition[0] - worldPos.xyz;
	float distanceToLight = length(lightDirection[0]);
	lightIntensity[0] = lightColor[0]*(1/(attenuation[0].x + attenuation[0].y*distanceToLight + attenuation[0].z*distanceToLight*distanceToLight));
	
	lightDirection[1] = lightPosition[1] - worldPos.xyz;
	distanceToLight = length(lightDirection[1]);
	lightIntensity[1] = lightColor[1]*(1/(attenuation[1].x + attenuation[1].y*distanceToLight + attenuation[1].z*distanceToLight*distanceToLight));

	lightDirection[2] = lightPosition[2] - worldPos.xyz;
	distanceToLight = length(lightDirection[2]);
	lightIntensity[2] = lightColor[2]*(1/(attenuation[2].x + attenuation[2].y*distanceToLight + attenuation[2].z*distanceToLight*distanceToLight));
	
	gl_Position = mvpMatrix * in_position;

}
